@@ -107,7 +107,7 @@ connect_to_server (CamelService *service,
 	CamelNetworkSettings *network_settings;
 	CamelNetworkSecurityMethod method;
 	CamelSettings *settings;
-	CamelStream *stream;
+	CamelStream *stream = NULL;
 	CamelPOP3Engine *pop3_engine = NULL;
 	CamelPOP3Command *pc;
 	GIOStream *base_stream;
@@ -219,7 +219,7 @@ connect_to_server (CamelService *service,
 		goto stls_exception;
 	}
 
-	g_object_unref (stream);
+	g_clear_object (&stream);
 
 	/* rfc2595, section 4 states that after a successful STLS
 	 * command, the client MUST discard prior CAPA responses */
@@ -242,8 +242,7 @@ stls_exception:
 	}*/
 
 exception:
-	g_object_unref (stream);
-
+	g_clear_object (&stream);
 	g_clear_object (&pop3_engine);
 
 	success = FALSE;
@@ -341,10 +340,10 @@ try_sasl (CamelPOP3Store *store,
 		if (strncmp ((gchar *) line, "+ ", 2) != 0
 		    || camel_sasl_get_authenticated (sasl)
 		    || (resp = (guchar *) camel_sasl_challenge_base64_sync (sasl, (const gchar *) line + 2, cancellable, &local_error)) == NULL) {
-			camel_stream_write_string (
-				CAMEL_STREAM (pop3_stream), "*\r\n", cancellable, NULL);
-			/* coverity[unchecked_value] */
-			camel_pop3_stream_line (pop3_stream, &line, &len, cancellable, NULL);
+			if (camel_stream_write_string (CAMEL_STREAM (pop3_stream), "*\r\n", cancellable, NULL)) {
+				/* coverity[unchecked_value] */
+				camel_pop3_stream_line (pop3_stream, &line, &len, cancellable, NULL);
+			}
 
 			if (local_error) {
 				g_propagate_error (error, local_error);
@@ -523,7 +522,7 @@ pop3_store_connect_sync (CamelService *service,
 
 	g_object_unref (settings);
 
-	if (!camel_session_get_online (session)) {
+	if (!session || !camel_session_get_online (session)) {
 		g_set_error (
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_UNAVAILABLE,
@@ -611,12 +610,15 @@ pop3_store_disconnect_sync (CamelService *service,
 		pop3_engine = camel_pop3_store_ref_engine (store);
 
 		if (pop3_engine) {
-			pc = camel_pop3_engine_command_new (
-				pop3_engine, 0, NULL, NULL,
-				cancellable, error, "QUIT\r\n");
-			while (camel_pop3_engine_iterate (pop3_engine, NULL, cancellable, NULL) > 0)
-				;
-			camel_pop3_engine_command_free (pop3_engine, pc);
+			if (camel_pop3_engine_busy_lock (pop3_engine, cancellable, NULL)) {
+				pc = camel_pop3_engine_command_new (
+					pop3_engine, 0, NULL, NULL,
+					cancellable, error, "QUIT\r\n");
+				while (camel_pop3_engine_iterate (pop3_engine, NULL, cancellable, NULL) > 0)
+					;
+				camel_pop3_engine_command_free (pop3_engine, pc);
+				camel_pop3_engine_busy_unlock (pop3_engine);
+			}
 
 			g_clear_object (&pop3_engine);
 		}
@@ -672,6 +674,14 @@ pop3_store_authenticate_sync (CamelService *service,
 		goto exit;
 	}
 
+	if (!camel_pop3_engine_busy_lock (pop3_engine, cancellable, error)) {
+		g_free (host);
+		g_free (user);
+		g_clear_object (&pop3_engine);
+
+		return CAMEL_AUTHENTICATION_ERROR;
+	}
+
 	if (mechanism == NULL) {
 		if (password == NULL) {
 			g_set_error_literal (
@@ -699,6 +709,14 @@ pop3_store_authenticate_sync (CamelService *service,
 			pop3_engine, 0, NULL, NULL, cancellable, error,
 			"PASS %s\r\n", password);
 
+		if (error && *error) {
+			g_prefix_error (
+				error,
+				_("Unable to connect to POP server %s.\n"
+				"Error sending password: "), host);
+			result = CAMEL_AUTHENTICATION_ERROR;
+			goto exit;
+		}
 	} else if (strcmp (mechanism, "+APOP") == 0 && pop3_engine->apop) {
 		gchar *secret, *md5asc, *d;
 		gsize secret_len;
@@ -803,15 +821,17 @@ pop3_store_authenticate_sync (CamelService *service,
 		result = CAMEL_AUTHENTICATION_ACCEPTED;
 	}
 
-	camel_pop3_engine_command_free (pop3_engine, pcp);
+exit:
+	if (pcp != NULL)
+		camel_pop3_engine_command_free (pop3_engine, pcp);
 
 	if (pcu != NULL)
 		camel_pop3_engine_command_free (pop3_engine, pcu);
 
-exit:
 	g_free (host);
 	g_free (user);
 
+	camel_pop3_engine_busy_unlock (pop3_engine);
 	g_clear_object (&pop3_engine);
 
 	return result;
@@ -1086,6 +1106,11 @@ camel_pop3_store_expunge (CamelPOP3Store *store,
 
 	pop3_engine = camel_pop3_store_ref_engine (store);
 
+	if (!camel_pop3_engine_busy_lock (pop3_engine, cancellable, error)) {
+		g_clear_object (&pop3_engine);
+		return FALSE;
+	}
+
 	pc = camel_pop3_engine_command_new (
 		pop3_engine, 0, NULL, NULL, cancellable, error, "QUIT\r\n");
 
@@ -1094,6 +1119,7 @@ camel_pop3_store_expunge (CamelPOP3Store *store,
 
 	camel_pop3_engine_command_free (pop3_engine, pc);
 
+	camel_pop3_engine_busy_unlock (pop3_engine);
 	g_clear_object (&pop3_engine);
 
 	return TRUE;
